#!/usr/bin/python3
#  -*- coding: utf-8 -*-

import re

import pprint

from utils import ValueError

def parse_value( value ) :
    result = Getter()

    quoted = re.compile( "^\"(.+)\"$" )
    integer = re.compile( "^((-|\+)?[0-9]+)$" )
    number = re.compile( "^((-|\+)?[0-9]*\.?[0-9]+)$" )
    exponent = re.compile( "^((-|\+)?[0-9]*\.?[0-9]+[eE](-|\+)?[0-9]+)$" )

    if result.set(quoted.match(value)) :
        parsed = result.val.group( 1 )
    elif result.set(integer.match(value)) :
        parsed = int( result.val.group(1) )
    elif result.set(number.match(value)) or result.set(exponent.match(value)) :
        parsed = float( result.val.group(1) )
    else :
        parsed = value

    return parsed

class Getter :
    def __init__( self ) :
        self.val = None

        return

    def set( self, val ) :
        self.val = val

        return val

class Cluster :
    def __init__( self, cid, parts ) :
        self.cid = None

        if len(parts) == 19 :
            parts = [ parse_value(pp) for pp in parts ]
            
            self.name = parts[0]
            self.bary = ( parts[1], parts[2] )
            self.utm  = ( parts[3], parts[4] )
            self.lonlat = ( parts[5], parts[6] )
            self.rpix = ValueError( parts[7], parts[8] )
            self.rmet = ValueError( parts[9], parts[10] )
            self.parea = parts[11]
            self.marea = parts[12]
            self.tmean = ValueError( parts[13], parts[14] )
            self.power = ValueError( parts[15], parts[16] )
            self.flux = ValueError( parts[17], parts[18] )

            self.cid = cid
            self.fid = self.prod = None

            self.date = self.epoch = None

        return

    def __repr__( self ) :
        return pprint.pformat( vars(self) )

    def inside( self, bbox ) :
        return (self.utm[0] >= bbox[0]) and (self.utm[0] <= bbox[1]) and (self.utm[1] >= bbox[2]) and (self.utm[1] <= bbox[3])

    def outside( self, bbox ) :
        return (self.utm[0] < bbox[0]) or (self.utm[0] > bbox[1]) or (self.utm[1] < bbox[2]) or (self.utm[1] > bbox[3])        

    def set_date( self, date, epoch ) :
        self.date = date
        self.epoch = epoch

        return self

    def set_fid( self, fid ) :
        self.fid = fid

        return self

    def set_product( self, prod ) :
        self.prod = int( prod )

        return self

    def get_full_name( self ) :
        if self.fid is None :
            cid = self.name
        else :
            cid = "{:03d}:{}".format(self.fid, self.name)

        return cid

    def good( self ) :
        return self.cid is not None

class DatFile :
    counter = 0
    
    def __init__( self, filename ) :
        self.filename = filename
        self.data = dict( fid = DatFile.counter )

        DatFile.counter += 1

        return

    def _read_header( self, handle, number, what, pieces ) :
        line = next( handle )
        values = line.split()

        if len(values) == (len(pieces) + 1) :            
            for i in range(len(pieces)) :
                self.data[pieces[i]] = parse_value( values[i+1] )
        else :
            print( "Different number of fields and fields found for header n.{}".format(number) )
            number = -1

        return number

    def read( self, bbox=None ) :
        data = None
        header = 1
        hexpr = re.compile( "^# ?H([0-9]+) (.+) - (.+)$" )
        datexpr = re.compile( "^# D[0-9]+ .+$" )
        clexpr = re.compile( "^C([0-9]+) (.+)$" )
        comment = re.compile( "^\s*#.*$" )

        with open( self.filename, "r" ) as handle :
            for line in handle :
                if header == 1 :
                    result = hexpr.match( line )
                    if result :
                        number = int( result.group(1) )
                        what = result.group( 2 )
                        pieces = result.group( 3 ).split()

                        header = self._read_header( handle, number, what, pieces )
                        if header == -1 :
                            break
                        else :
                            header = 1
                            continue

                    result = datexpr.match( line )
                    if result :
                        header = 0
                        continue

                elif header == 0 :
                    result = clexpr.match( line )

                    if result :
                        cluster = Cluster( int(result.group(1)), result.group(2).split() )
                        cluster.set_fid( self.data['fid'] ).set_product( self.data['product'] )

                        if (bbox is not None) and cluster.outside(bbox) :
                            #print( "Cluster at {} discarded ({})".format(cluster.utm, ", ".join([ str(x) for x in bbox])) )
                            continue

                        if 'clusters' in self.data :
                            self.data['clusters'].append( cluster )
                        else :
                            self.data['clusters'] = [ cluster ]
                    elif not comment.match(line) :
                        print( "Cluster parsing error!" )
                        header = -1
                        break

        #pprint.pprint( self.data )

        if header != -1 :
            data = self.data

        return data
