from libreria import *
import sys
import math
import re
import argparse, os
from os import walk
import matplotlib.pyplot as plt
from pathlib import Path
import datetime
import numpy as np
import pandas as pd
from sklearn.cluster import DBSCAN
from sklearn import metrics
from sklearn.datasets import make_blobs
from sklearn.preprocessing import StandardScaler


########################################################################
#
# Il programma ed in particolare la clusterizzazione è molto sensibile
# a due parametri:
#
#  -> eps : è la distanza entro la quale due punti sono considerati
#     dello stesso cluster (di default è uguale a 20);
#
#  -> min_samples : è il numero minimo di punti per formare un cluster (di default
#                   è uguale a 200)
#
########################################################################



parser = argparse.ArgumentParser(description='Il programma fornisce una stima del numero di cluster')

parser.add_argument("-i", "--input",
                    nargs='+',
                    help='Path della directory di input contenente i file .csv',
                    required=True)

parser.add_argument("-o", "--output",
                    help='Path della directory di output in cui salvare i file ascii',
                    required=True)

parser.add_argument("-s", "--savefig",
                    help='Spegne il display a video e salva le immagini in \
una directory (richiede di specificare una directory)',
                    required=False
)

parser.add_argument("--statistics",
                    help='Spegne il display a video e salva i grafici \
delle statistiche in una directory (richiede di specificare una directory)',
                    required=False
)

parser.add_argument("-e", "--eps",
                    help='eps è la distanza entro la quale due punti sono \
                        considerati dello stesso cluster (default: 20)',
                    required=False,
                    default=20
)

parser.add_argument("-m", "--minSamples",
                    help='minSamples è il numero minimo di punti per formare \
un cluster (default: 200)',
                    required=False,
                    default=200
)

parser.add_argument("-k", "--kmeans",
                    help='k è il numero di cluster che si vuole ottenere \
dalla segmentazione dell\'immagine tramite il kmeans(default: 5)',
                    required=False,
                    default=5
)

args = parser.parse_args()

input_path = []
input_path.extend(args.input) # input_path è una lista di path di directory

output_path = args.output

eps = float(args.eps)
minSamples = float(args.minSamples)
k = int(args.kmeans)


saveStatDir = args.statistics
if saveStatDir is None:
    statShow = True  # show
else:
    statShow = False # save

saveFigDir = args.savefig
if saveFigDir is None:
    dispOff = False  # show
else:
    dispOff = True # save



f = []
# Se in input è stato inserito il percorso di una o più directory
if os.path.isdir(input_path[0]): 
    for i in range(len(input_path)):
        for (dirpath, dirnames, filenames) in walk(input_path[i]):
            for j in range(len(filenames)):
                if re.search('\.csv', filenames[j]):
                    f.append(os.path.join(input_path[i],filenames[j]))
                # f contiene i path di tutti i file delle directory in input
                
else: # se in input è stato inserito il percorso di un solo file
    if re.search('\.csv', input_path[0]):
        f.append(input_path[0])
    else:
        print('Not .csv file')


dict_temp = {}
dict_pixel = {}
dict_stdev = {}
dict_date = {}


for i in range(len(f)):

    file_path = f[i]
    
    p = Path(file_path)
    
    fileName = p.name # Estrae il nome del file dal Path
    
    print("Processando file n." + str(i) + "   nome: " + fileName)

    sep = '_'
    areaName = re.split(sep, fileName, 1)[0] # Ottiene il nome dell'area

    data = re.split("_", fileName)[1] # Ottiene la data del file (la quale è compresa tra due underscore)
    data = re.sub('[a-zA-Z]', '', data) # Elimina eventuali lettere dalla data


    # Ottiene il formato mese/anno da porre sui grafici delle statistiche finali
    date_split = datetime.datetime.strptime(data, "%Y%m%d")
    graph_date = date_split.strftime("%m/%Y") #formato  mese/anno

    
    matrix, nRow, nColumn = leggiMatrice(file_path) # Legge un file csv

    cord, minPixel, maxPixel = get3d(matrix, nRow, nColumn)
    

    cluster_dict = opencv(k, minPixel, maxPixel, matrix, \
                          nRow, nColumn, saveFigDir, i, dispOff, eps, minSamples)
    n_clusters_ = len(cluster_dict)
    info, hottest_pixel, hottest_temp, stdevHottest = getInfoClusters(n_clusters_, cluster_dict)
    writeFile(areaName, data, n_clusters_, info, output_path)


    # Dizionario che contiene le temperature medie dei 3 cluster più caldi
    # di ogni AREA
    # nome_area: list(temp1,temp2,...)
    dict_temp.setdefault(areaName, []) # se areaName non esiste, inizializza l'array relativo alla chiave areaName
    dict_temp[areaName].append(hottest_temp)

    # Dizionario che contiene la deviazione standard dei 3 cluster più caldi
    dict_stdev.setdefault(areaName, [])
    dict_stdev[areaName].append(stdevHottest)

    # Dizionario che contiene il numero medio dei pixel dei 3 cluster più caldi
    dict_pixel.setdefault(areaName, [])
    dict_pixel[areaName].append(hottest_pixel)

    # Dizionario che contiene le date delle immagini (da porre sui grafici delle statistiche finali )
    dict_date.setdefault(areaName, [])
    dict_date[areaName].append(graph_date)


for i in dict_temp:
    
    # Grafico andamento temperatura
    fig_temp = plt.figure(figsize=(11, 6))
    plt.title('Area: ' + i)
    #plt.xlabel('Data dell\'immagine')
    plt.ylabel('Temperatura media dei cluster più caldi (°C)') 
    plt.grid()
    plt.errorbar(dict_date[i], dict_temp[i], yerr = dict_stdev[i], marker='o', capsize = 5) 
    plt.xticks(fontsize=8.5, rotation=-90)
    
    if statShow == True:
        plt.show()
    else:   
        fig_temp.savefig(saveStatDir + "\\" + i + '_temperature' +'.png', dpi=fig_temp.dpi)
    plt.close(fig_temp)


    
    # Grafico andamento numero di pixel
    fig_pixel = plt.figure(figsize=(11, 6))
    plt.title('Area: ' + i)
    #plt.xlabel('Immagine')
    plt.ylabel('Numero medio di pixel dei cluster più caldi')
    plt.grid()
    plt.plot(dict_date[i], dict_pixel[i], marker='s')
    plt.xticks(fontsize=8.5, rotation=-90)

    if statShow == True:
        plt.show()
    else: 
        fig_pixel.savefig(saveStatDir + "\\" + i + '_pixel' + '.png', dpi=fig_pixel.dpi)
    plt.close(fig_pixel)




  
