#!/usr/bin/python3
#  -*- coding: utf-8 -*-

import sys

import pyproj
import matplotlib.pyplot as plt

import asciigrid
from utils import Pixel

grid = asciigrid.AsciiGrid( sys.argv[1], True )
if grid.good :
    #proj = pyproj.Proj( "+proj=utm +zone=33t, +north +ellps=WGS84 +datum=WGS84 +units=m +no_defs" )
    proj = pyproj.Proj( proj="utm", zone=33, ellpl="WGS84", datum="WGS84", units="m" )

    print( "File read: ({} x {}) pixels, ({:.1f} x {:.1f}) m, min: {:.1f}, max: {:.1f}".format(grid.rows, grid.columns, grid.width(), grid.height(), grid.minimum, grid.maximum) )
    print( "LP ({:.1f}, {:.1f}), HP ({:.1f} {:.1f})".format(grid.xllcorner, grid.yllcorner, grid.xllcorner + grid.width(), grid.yllcorner + grid.height()) )

    lxy = Pixel( 0, 0 )
    utm = grid.pixel2utm( lxy )
    px  = grid.utm2pixel( utm.pixel )
    lonlat = proj( utm.x(), utm.y(), inverse=True )
    print( "Pt {} => {} ({:.8f}, {:.8f}) => {}".format(lxy, utm, *lonlat, px) )

    lxy = Pixel( grid.columns, grid.rows )
    utm = grid.pixel2utm( lxy )
    px  = grid.utm2pixel( utm.pixel )
    lonlat = proj( utm.x(), utm.y(), inverse=True )
    print( "Pt {} => {} ({:.8f}, {:.8f}) => {}".format(lxy, utm, *lonlat, px) )
    
    grid.draw()
    plt.show()
else :
    print( "Error reading file <{}>".format(sys.argv[1]) )
    print( grid.error )
